/*
 * (C) 2007, Tom Wheeler.  All rights reserved.
 */
package com.tomwheeler.demo.jcr.worldinfo;

import java.io.*;

import javax.jcr.*;
import javax.naming.*;
import org.apache.jackrabbit.core.TransientRepository;
import org.apache.jackrabbit.core.config.RepositoryConfig;
import org.apache.commons.io.FileUtils;


/**
 * <p>
 * Convenience class for working with the Java Content Repository in the
 * WorldInfo application.  Client data operations, aside from getting a 
 * session to the repository itself, should actually be done with the 
 * CountryManager class.  Most of the code in RepoManager is specific
 * to the Jackrabbit implementation.
 * </p>
 * 
 * <p>
 * This implementation is intended to generally illustrate the
 * simplest possible approach for working with an embedded Jackrabbit
 * content repository for an application under development.  Techniques
 * on creating, accessing and shutting down the repository will vary 
 * for other JCR implementations, for remote repositories and for 
 * applications that wish to abstract some of these details by using JNDI.
 * Consult your vendor's documentation for details on what is appropriate
 * for your case.
 * </p>
 * 
 * @author Tom Wheeler
 */
public class RepoManager {

    /*
     * You can arrange your repository however you like.  I am using
     * the following layout, embedded within the project:
     *
     *   + repository
     *       |    
     *       + config
     *       |   + jaas.config
     *       |   + repository.xml
     *       |
     *       + data
     *           + logs
     *           + storage
     */
    
    private static final String REPO_PATH = "repository";    

    private static final String REPO_CFG_PATH = REPO_PATH  + "/config";    
    private static final String REPO_CFG_FILE = REPO_CFG_PATH  + "/repository.xml";
    private static final String JAAS_CONFIG_PATH = REPO_CFG_PATH + "/jaas.config";
    
    private static final String DATA_PATH = REPO_PATH + "/data";    
    private static final String REPO_STORAGE_PATH = DATA_PATH  + "/storage";
    private static final String LOG_PATH = DATA_PATH  + "/logs";

    private static Repository repo;    
    private static Session session;
    
    public static void deleteRepository() throws IOException {
        // There is no standard API call for deleting a repository, but
        // it's handy for writing tests which begin from a known state.
        // My implementation is a bit of a hack; since I happen to know 
        // the location of the repository, I can remove it by just deleting 
        // its directory.  Whenever the repository is requested again with
        // the getRepository method, a new (empty) one will be recreated.
        // FileUtils.deleteDirectory is a Jakarta Commons IO utility method
        // for recursively removing a directory
        
        File dataPathAsFile = new File(DATA_PATH);
        if (dataPathAsFile.exists()) {            
            FileUtils.deleteDirectory(dataPathAsFile);
        }
    }
        
    private synchronized static Repository getRepository() { 
        if (repo == null) {
            
            try {
                File dataPathAsFile = new File(".", DATA_PATH);
                if (!dataPathAsFile.exists()) {
                    dataPathAsFile.mkdir();
                }

                System.setProperty("java.security.auth.login.config", JAAS_CONFIG_PATH); 
                System.setProperty("derby.system.home", LOG_PATH);

                RepositoryConfig cfg = RepositoryConfig.create(REPO_CFG_FILE, REPO_STORAGE_PATH);
                repo = new TransientRepository(cfg);

            } catch (IOException ex) {
                String msg = "Caught IOException while creating repository";
                throw new DataAccessException(msg, ex);
            } catch (org.apache.jackrabbit.core.config.ConfigurationException ex) {
                String msg = "Caught ConfigurationException while creating repository";
                throw new DataAccessException(msg, ex);
            }
        }

        return repo;
    }
    
    public synchronized static Session getSession() {
        if (session == null) {
            try {
                // The default Jackrabbit login mechanism accepts any username 
                // and password as valid credentials and returns a session with 
                // full write access.  Using the no-argument version of login
                // here would return a session with read-only access to the 
                // repository.
                SimpleCredentials cred = 
                        new SimpleCredentials("userid", "password".toCharArray());

                session = getRepository().login(cred);
            } catch (LoginException ex) {
                String msg = "Caught LoginException while getting session";
                throw new DataAccessException(msg, ex);
            } catch (RepositoryException ex) {
                String msg = "Caught RepositoryException while getting session";
                throw new DataAccessException(msg, ex);
            }
        }
        
        return session;
    }
}
