/*
 * (C) 2007, Tom Wheeler.  All rights reserved.
 */

package com.tomwheeler.demo.jcr.worldinfo.gui;

import com.tomwheeler.demo.jcr.worldinfo.Country;
import com.tomwheeler.demo.jcr.worldinfo.CountryManager;
import com.tomwheeler.demo.jcr.worldinfo.DataAccessException;
import com.tomwheeler.demo.jcr.worldinfo.RepoManager;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import javax.jcr.Session;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * <p>
 * This is the controller class for the MainPanel GUI.  The Swing
 * application logic (such as listeners) is in this class.  There
 * is actually no JCR code directly in this class; it simply calls
 * methods in the CountryManager class.
 * </p>
 * 
 * <p>
 * NOTE: In this application, I am creating a new repository and populating
 * it each time the program is run.  In a less academic example, you would
 * want to take care to call logout() on your session and to shut down the
 * repository to ensure data integrity.
 * </p>
 */
public class AppController {

    private MainPanel mainPanel;
    private Session session;
    private CountryManager manager;
    private InfoChangedListener icListener;
    private SaveActionListener saveActionListener;
    private FindActionListener findActionListener;
    private AddCountryActionListener addCountryActionListener;

    public static void main(String[] args) throws DataAccessException {
        AppController ac = new AppController();
        ac.start();
    }

    public AppController() {
        try {
            RepoManager.deleteRepository();
        } catch (IOException ex) {
            String msg = "Caught IOException while deleting old repository: ";
            msg += "'" + ex.getMessage() + "'";

            JOptionPane.showMessageDialog(mainPanel, msg);
        }

        mainPanel = new MainPanel();
        session = RepoManager.getSession();
        manager = new CountryManager(session);

        mainPanel.getCountryList().setModel(new DefaultListModel());
        
        icListener = new InfoChangedListener();
        saveActionListener = new SaveActionListener();
        findActionListener = new FindActionListener();
        addCountryActionListener = new AddCountryActionListener();
    }

    public void start() throws DataAccessException {
        createAndPopulate(manager);

        setCountrySelected(null);

        mainPanel.getNameField().getDocument().addDocumentListener(icListener);
        mainPanel.getContinentField().getDocument().addDocumentListener(icListener);
        mainPanel.getCurrencyField().getDocument().addDocumentListener(icListener);
        mainPanel.getPopulationField().getDocument().addDocumentListener(icListener);
        mainPanel.getPrincipalLanguageField().getDocument().addDocumentListener(icListener);
        mainPanel.getDateFormedDay().getDocument().addDocumentListener(icListener);
        mainPanel.getDateFormedMonth().getDocument().addDocumentListener(icListener);
        mainPanel.getDateFormedYear().getDocument().addDocumentListener(icListener);
        mainPanel.getTldField().getDocument().addDocumentListener(icListener);

        final JList guiList = mainPanel.getCountryList();
        guiList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        guiList.setCellRenderer(new CountryListRenderer());
        guiList.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }

                Country selCountry = (Country) guiList.getSelectedValue();
                setCountrySelected(selCountry);
            }
        });
        
        mainPanel.getSaveButton().addActionListener(saveActionListener);
        mainPanel.getCancelButton().addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // will set the fields back to their original values
                Country selected = (Country)guiList.getSelectedValue();
                setCountrySelected(selected);
            }
        });

        rebuildCountryList();

        mainPanel.getDeleteButton().addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Country selCountry = (Country) guiList.getSelectedValue();
                manager.delete(selCountry);
                rebuildCountryList();
            }
        });

        mainPanel.getFindButton().addActionListener(findActionListener);
        mainPanel.getAddButton().addActionListener(addCountryActionListener);

        JFrame frame = new JFrame("WorldInfo Application");
        Container pane = frame.getContentPane();
        pane.setLayout(new BorderLayout());
        pane.add(mainPanel, BorderLayout.CENTER);

        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }

    private void rebuildCountryList() {
        JList guiList = mainPanel.getCountryList();
        guiList.setCellRenderer(new CountryListRenderer());

        DefaultListModel guiListModel = (DefaultListModel) guiList.getModel();
        guiListModel.clear();

        List<Country> countries = manager.getAllCountries();
        for (Country country : countries) {
            guiListModel.addElement(country);
        }
    }

    private void setCountrySelected(Country country) {
        if (country == null) {
            mainPanel.getCancelButton().setEnabled(false);
            mainPanel.getSaveButton().setEnabled(false);
            mainPanel.getDeleteButton().setEnabled(false);

            mainPanel.getNameField().setText(null);
            mainPanel.getContinentField().setText(null);
            mainPanel.getCurrencyField().setText(null);
            mainPanel.getPopulationField().setText(null);
            mainPanel.getPrincipalLanguageField().setText(null);
            mainPanel.getDateFormedDay().setText(null);
            mainPanel.getDateFormedMonth().setText(null);
            mainPanel.getDateFormedYear().setText(null);
            mainPanel.getTldField().setText(null);
        } else {
            mainPanel.getCancelButton().setEnabled(true);
            mainPanel.getSaveButton().setEnabled(true);
            mainPanel.getDeleteButton().setEnabled(true);

            mainPanel.getNameField().setText(country.getName());
            mainPanel.getContinentField().setText(country.getContinent());
            mainPanel.getCurrencyField().setText(country.getCurrency());
            mainPanel.getPopulationField().setText("" + country.getPopulation());
            mainPanel.getPrincipalLanguageField().setText(country.getPrincipalLanguage());
            mainPanel.getTldField().setText(country.getTld());

            String yearValue = null;
            String monthValue = null;
            String dayValue = null;

            Calendar dfCal = country.getDateFormed();
            if (dfCal != null) {
                yearValue = "" + (dfCal.get(Calendar.YEAR));
                monthValue = "" + (dfCal.get(Calendar.MONTH) + 1);
                dayValue = "" + dfCal.get(Calendar.DAY_OF_MONTH);
            }

            mainPanel.getDateFormedDay().setText(dayValue);
            mainPanel.getDateFormedMonth().setText(monthValue);
            mainPanel.getDateFormedYear().setText(yearValue);
        }
        
        setSaveEnabled(false);
    }

    public static void createAndPopulate(CountryManager manager) throws DataAccessException {

        Country usa = new Country("USA", "North America");
        usa.setCurrency("Dollar");
        usa.setDateFormed(new GregorianCalendar(1776, 6, 4));
        usa.setPrincipalLanguage("English");
        usa.setPopulation(300000000);
        usa.setTld(".us");
        manager.add(usa);

        Country argentina = new Country("Argentina", "South America");
        argentina.setCurrency("Peso");
        argentina.setDateFormed(new GregorianCalendar(1816, 6, 16));
        argentina.setPrincipalLanguage("Spanish");
        argentina.setPopulation(40000000);
        argentina.setTld(".ar");
        manager.add(argentina);

        Country brazil = new Country("Brazil", "South America");
        brazil.setCurrency("Real");
        brazil.setDateFormed(new GregorianCalendar(1822, 8, 7));
        brazil.setPrincipalLanguage("Portuguese");
        brazil.setPopulation(190000000);
        brazil.setTld(".br");
        manager.add(brazil);

        Country canada = new Country("Canada", "North America");
        canada.setCurrency("Dollar");
        canada.setDateFormed(new GregorianCalendar(1867, 6, 1));
        canada.setPrincipalLanguage("English");
        canada.setPopulation(190000000);
        canada.setTld(".ca");
        manager.add(canada);

        Country china = new Country("China", "Asia");
        china.setCurrency("Yuan");
        china.setDateFormed(new GregorianCalendar(1949, 9, 1));
        china.setPrincipalLanguage("Mandarin");
        china.setPopulation(1300000000);
        china.setTld(".cn");
        manager.add(china);

        Country egypt = new Country("Egypt", "Africa");
        egypt.setCurrency("Pound");
        egypt.setDateFormed(new GregorianCalendar(1922, 1, 28));
        egypt.setPrincipalLanguage("Arabic");
        egypt.setPopulation(80000000);
        egypt.setTld(".eg");
        manager.add(egypt);

        Country germany = new Country("Germany", "Europe");
        germany.setCurrency("Euro");
        germany.setDateFormed(new GregorianCalendar(1990, 9, 3));
        germany.setPrincipalLanguage("German");
        germany.setPopulation(82000000);
        germany.setTld(".de");
        manager.add(germany);

        Country india = new Country("India", "Asia");
        india.setCurrency("Rupee");
        india.setDateFormed(new GregorianCalendar(1947, 7, 15));
        india.setPrincipalLanguage("Hindi");
        india.setPopulation(1120000000);
        india.setTld(".in");
        manager.add(india);

        Country japan = new Country("Japan", "Asia");
        japan.setCurrency("Yen");
        japan.setDateFormed(new GregorianCalendar(1947, 4, 3));
        japan.setPrincipalLanguage("Japanese");
        japan.setPopulation(127000000);
        japan.setTld(".jp");
        manager.add(japan);

        Country mexico = new Country("Mexico", "North America");
        mexico.setCurrency("Peso");
        mexico.setDateFormed(new GregorianCalendar(1810, 8, 16));
        mexico.setPrincipalLanguage("Spanish");
        mexico.setPopulation(108000000);
        mexico.setTld(".mx");
        manager.add(mexico);

        Country mozambique = new Country("Mozambique", "Africa");
        mozambique.setCurrency("Metical");
        mozambique.setDateFormed(new GregorianCalendar(1975, 5, 25));
        mozambique.setPrincipalLanguage("Portuguese");
        mozambique.setPopulation(20000000);
        mozambique.setTld(".mz");
        manager.add(mozambique);

        Country thailand = new Country("Thailand", "Asia");
        thailand.setCurrency("Baht");
        thailand.setDateFormed(new GregorianCalendar(1782, 3, 6));
        thailand.setPrincipalLanguage("Thai");
        thailand.setPopulation(60000000);
        thailand.setTld(".th");
        manager.add(thailand);

        Country uk = new Country("United Kingdom", "Europe");
        uk.setCurrency("Pound");
        uk.setDateFormed(new GregorianCalendar(1707, 4, 1));
        uk.setPrincipalLanguage("English");
        uk.setPopulation(60000000);
        uk.setTld(".uk");
        manager.add(uk);
    }
    
    private void setSaveEnabled(boolean enabled) {
        mainPanel.getSaveButton().setEnabled(enabled);
        mainPanel.getCancelButton().setEnabled(enabled);
    }

    private class InfoChangedListener implements DocumentListener {

        public void insertUpdate(DocumentEvent e) {
            setSaveEnabled(true);
        }

        public void removeUpdate(DocumentEvent e) {
            setSaveEnabled(true);
        }

        public void changedUpdate(DocumentEvent e) {
            setSaveEnabled(true);
        }
    }
    
    private class FindActionListener implements ActionListener {
        
    
        public void actionPerformed(ActionEvent e) {

            String msg = "Enter Internet TLD:";
            String tld = JOptionPane.showInputDialog(mainPanel, msg);
            if (tld != null && tld.length() > 0) {

                Country match = null;
                try {
                    match = manager.findByTld(tld);
                } catch (DataAccessException dae) {
                    // ignore; will be detected by null match
                }

                if (match == null) {
                    msg = "No country found with TLD ='" + tld + "'";
                } else {
                    msg = "Country " + match.getName() + " has TLD ='" + tld + "'";
                }

                JOptionPane.showMessageDialog(mainPanel, msg);
            }
        }
    }
    
    private class AddCountryActionListener implements ActionListener {
        
    
        public void actionPerformed(ActionEvent e) {

            // I am making the GUI here as simple as possible.  Instead of 
            // asking user to fill out all the values at once, I ask for the 
            // name and continent (required to build the path), then use
            // temporary values for everything else which the user can then
            // change
            String contMsg = "Enter Continent Name:";
            String continent = JOptionPane.showInputDialog(mainPanel, contMsg);
            if (continent != null && continent.length() > 0) {

                String countryMsg = "Enter Country Name:";
                String countryName = JOptionPane.showInputDialog(mainPanel, countryMsg);
                
                if (countryName != null && countryName.length() > 0) {
                    
                    Country newCountry = new Country(countryName, continent);
                    newCountry.setCurrency("temp");
                    newCountry.setDateFormed(new GregorianCalendar(2000, 0, 1));
                    newCountry.setPopulation(999L);
                    newCountry.setPrincipalLanguage("temp");
                    newCountry.setTld("temp");
                    
                    try {
                        manager.add(newCountry);
                        
                        JList guiList = mainPanel.getCountryList();
                        DefaultListModel guiListModel = (DefaultListModel) guiList.getModel();
                        guiListModel.addElement(newCountry);
                        guiList.setSelectedIndex(guiListModel.getSize() - 1);
                        setCountrySelected(newCountry);
                        
                        JOptionPane.showMessageDialog(mainPanel, 
                                "Added new country; please edit the values and save.");
                    } catch (DataAccessException dae) {
                        JOptionPane.showMessageDialog(mainPanel, 
                                "Add failed '" + dae.getMessage() + "'");
                    }
                }
            }
        }
    }

    private class SaveActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {

            String name = mainPanel.getNameField().getText();
            String continent = mainPanel.getContinentField().getText();

            Country country = new Country(name, continent);
            country.setCurrency(mainPanel.getCurrencyField().getText());
            country.setPrincipalLanguage(mainPanel.getPrincipalLanguageField().getText());
            country.setTld(mainPanel.getTldField().getText());

            long population = 0;
            try {
                String text = mainPanel.getPopulationField().getText();
                population = Long.parseLong(text);
                country.setPopulation(population);
            } catch (NumberFormatException nfe) {
                JOptionPane.showMessageDialog(mainPanel, "Invalid value for population field");
                return;
            }

            int year = 0;
            try {
                String text = mainPanel.getDateFormedYear().getText();
                year = Integer.parseInt(text);
            } catch (NumberFormatException nfe) {
                JOptionPane.showMessageDialog(mainPanel, "Invalid year for date formed field");
                return;
            }

            int month = 0;
            try {
                String text = mainPanel.getDateFormedMonth().getText();
                month = Integer.parseInt(text);
                month--;
            } catch (NumberFormatException nfe) {
                JOptionPane.showMessageDialog(mainPanel, "Invalid month for date formed field");
                return;
            }

            int day = 0;
            try {
                String text = mainPanel.getDateFormedDay().getText();
                day = Integer.parseInt(text);
            } catch (NumberFormatException nfe) {
                JOptionPane.showMessageDialog(mainPanel, "Invalid day for date formed field");
                return;
            }

            Calendar formed = new GregorianCalendar(year, month, day);
            country.setDateFormed(formed);

            try {
                manager.update(country);
                setSaveEnabled(false);
                
                // update the list model to reflect the change; saves a roundtrip
                // to the repository
                
                JList guiList = mainPanel.getCountryList();
                DefaultListModel guiListModel = (DefaultListModel) guiList.getModel();

                for (int i = 0; i < guiListModel.size(); i++) {
                    Country old = (Country)guiListModel.getElementAt(i);
                    if (name.equals(old.getName())) {
                        guiListModel.setElementAt(country, i);
                        break;
                    }
                }

                JOptionPane.showMessageDialog(mainPanel, "Information successfully saved");
            } catch (DataAccessException dae) {
                JOptionPane.showMessageDialog(mainPanel, "Update failed '" + dae.getMessage() + "'");
                return;
            }
        }
    }
}